import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';

import 'package:scrobbles/config/app_colors.dart';
import 'package:scrobbles/models/topartists.dart';
import 'package:scrobbles/ui/widgets/abstracts/custom_line_chart.dart';
import 'package:scrobbles/utils/color_extensions.dart';

class ChartTopArtistsStream extends CustomLineChart {
  final TopArtistsData chartData;

  const ChartTopArtistsStream({super.key, required this.chartData});

  final double verticalTicksInterval = 10;

  @override
  Widget build(BuildContext context) {
    final horizontalScale = getHorizontalScale();

    if (this.chartData.topArtistsStream.keys.isEmpty) {
      return SizedBox(
        height: this.chartHeight,
      );
    }

    return Container(
      height: this.chartHeight,
      child: LineChart(
        LineChartData(
          lineBarsData: getDataStreamLine(),
          betweenBarsData: getBetweenBarsData(),
          borderData: getBorderData(),
          gridData: getGridData(),
          titlesData: getTitlesData(),
          lineTouchData: const LineTouchData(enabled: false),
          minX: horizontalScale['min'],
          maxX: horizontalScale['max'],
          maxY: getNextRoundNumber(getMaxVerticalValue(), this.verticalTicksInterval),
          minY: 0,
        ),
        duration: const Duration(milliseconds: 250),
      ),
    );
  }

  double getNextRoundNumber(double number, double scale) {
    return scale * ((number ~/ scale).toInt() + 1);
  }

  FlGridData getGridData() {
    return const FlGridData(
      show: true,
      drawHorizontalLine: true,
      drawVerticalLine: false,
    );
  }

  double getMaxVerticalValue() {
    double maxValue = 0;

    this.chartData.topArtistsStream.keys.forEach((dateAsString) {
      double totalValue = 0.0;
      List<TopArtistsStreamDataValue> artists =
          this.chartData.topArtistsStream[dateAsString] ?? [];

      artists.forEach((artist) {
        final double value = artist.value;
        totalValue = totalValue + value;
      });

      if (totalValue > maxValue) {
        maxValue = totalValue;
      }
    });

    return maxValue;
  }

  Map<String, double> getHorizontalScale() {
    double minDateAsDouble = double.maxFinite;
    double maxDateAsDouble = -double.maxFinite;

    this.chartData.topArtistsStream.keys.forEach((dateAsString) {
      final double date = DateTime.parse(dateAsString).millisecondsSinceEpoch.toDouble();

      if (date < minDateAsDouble) {
        minDateAsDouble = date;
      }
      if (date > maxDateAsDouble) {
        maxDateAsDouble = date;
      }
    });

    return {
      'min': minDateAsDouble,
      'max': maxDateAsDouble,
    };
  }

  Color getColorIndex(int index) {
    const List<int> hexValues = [
      0x8dd3c7,
      0xffffb3,
      0xbebada,
      0xfb8072,
      0x80b1d3,
      0xfdb462,
      0xb3de69,
      0xfccde5,
      0xd9d9d9,
      0xbc80bd,
      0xccebc5,
      0xffed6f,
    ];

    return Color(hexValues[index % hexValues.length] + 0xff000000);
  }

  List<LineChartBarData> getDataStreamLine() {
    int artistsCount =
        this.chartData.topArtistsStream[this.chartData.topArtistsStream.keys.first]?.length ??
            0;

    List<LineChartBarData> lines = [];

    LineChartBarData getZeroHorizontalLine() {
      final baseColor = getColorIndex(0);
      final borderColor = baseColor.darken(20);

      List<FlSpot> spots = [];

      this.chartData.topArtistsStream.keys.forEach((dateAsString) {
        final double date = DateTime.parse(dateAsString).millisecondsSinceEpoch.toDouble();
        spots.add(FlSpot(date, 0));
      });

      return LineChartBarData(
        color: borderColor,
        dotData: const FlDotData(show: false),
        spots: spots,
      );
    }

    // First horizontal "zero" line
    lines.add(getZeroHorizontalLine());

    LineChartBarData getLinesFromIndex(int index) {
      final baseColor = getColorIndex(index);
      final borderColor = baseColor.darken(20);

      List<FlSpot> spots = [];

      this.chartData.topArtistsStream.keys.forEach((dateAsString) {
        final double date = DateTime.parse(dateAsString).millisecondsSinceEpoch.toDouble();

        List<TopArtistsStreamDataValue> artists =
            this.chartData.topArtistsStream[dateAsString] ?? [];

        double value = 0;
        for (int i = 0; i <= index; i++) {
          value = value + artists[i].value;
        }

        spots.add(FlSpot(date, value));
      });

      return LineChartBarData(
        isCurved: true,
        curveSmoothness: 0.25,
        color: borderColor,
        dotData: const FlDotData(show: false),
        spots: spots,
      );
    }

    Iterable<int>.generate(artistsCount)
        .toList()
        .map((index) => lines.add(getLinesFromIndex(index)))
        .toList();

    return lines;
  }

  List<BetweenBarsData> getBetweenBarsData() {
    int artistsCount =
        this.chartData.topArtistsStream[this.chartData.topArtistsStream.keys.first]?.length ??
            0;

    return Iterable<int>.generate(artistsCount)
        .toList()
        .map((index) => BetweenBarsData(
              fromIndex: index,
              toIndex: index + 1,
              color: getColorIndex(index),
            ))
        .toList();
  }

  Widget getHorizontalTitlesWidget(double value, TitleMeta meta) {
    final DateFormat formatter = DateFormat('dd/MM');

    final DateTime date = DateTime.fromMillisecondsSinceEpoch(value.toInt());
    final String text = formatter.format(date);

    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: 4,
      child: Padding(
        padding: EdgeInsets.only(right: 10),
        child: RotationTransition(
          turns: new AlwaysStoppedAnimation(-30 / 360),
          child: Text(
            text,
            style: TextStyle(
              color: AppColors.mainTextColor1,
              fontSize: this.titleFontSize,
            ),
          ),
        ),
      ),
    );
  }
}
